<?php

namespace App\Service;

use App\Entity\Author;
use App\Entity\Commit;
use Doctrine\ORM\EntityManagerInterface;
use Github\Client;

class GithubLoadingService
{
    private const NUMBER_OF_COMMITS = 25;

    private const PLATFORM_NAME = 'github';

    /**
     * @var EntityManagerInterface
     */
    private $entityManager;
    /**
     * @var Client
     */
    private $client;

    /**
     * GithubLoadingService constructor.
     * @param Client $client
     * @param EntityManagerInterface $entityManager
     */
    public function __construct(
        Client $client,
        EntityManagerInterface $entityManager
    ) {
        $this->client = $client;
        $this->entityManager = $entityManager;
    }

    /**
     * @param string $username
     * @param string $repoName
     * @return int
     */
    public function load(string $username, string $repoName): int
    {

        $commits = $this->client
            ->api('repo')
            ->commits()
            ->all($username, $repoName, array(
                'sha' => 'master',
                'per_page' => self::NUMBER_OF_COMMITS,
                'page' => 0
            ));
        $counter = 0;
        foreach ($commits as $commit) {
            if (!$this->commitExists($commit['sha'])) {
                $this->importCommit($commit);
                $counter++;
            }
        }

        return $counter;
    }

    /**
     * @param $sha
     * @return bool
     */
    private function commitExists($sha): bool
    {
        $repo = $this->entityManager->getRepository(Commit::class);
        $commit = $repo->findOneBy(
            [
                'sha' => $sha,
                'platform' => self::PLATFORM_NAME
            ]
        );
        return !is_null($commit);
    }

    /**
     * @param array $commitData
     */
    private function importCommit(array $commitData): void
    {
        print $commitData['sha'];
        print "\n";
        $author = $this->getAuthor($commitData['commit']['author'], $commitData['author']);
        $commit = new Commit();
        $commit->setPlatform(self::PLATFORM_NAME)
            ->setAuthor($author)
            ->setMessage($commitData['commit']['message'])
            ->setCommitDate(new \DateTime($commitData['commit']['committer']['date']))
            ->setSha($commitData['sha'])
            ->setUrl($commitData['url']);
        $this->entityManager->persist($commit);
        $this->entityManager->flush();
    }

    /**
     * @param array $commitAuthor
     * @param array $authorDetails
     * @return Author
     */
    private function getAuthor(array $commitAuthor, array $authorDetails): Author
    {
        $email = $commitAuthor['email'];
        $repo = $this->entityManager->getRepository(Author::class);
        $author = $repo->findOneBy(['email' => $email]);
        if ($author) {
            return $author;
        }
        $author = new Author();
        $author->setEmail($email)
            ->setAvatarUrl($authorDetails['avatar_url'])
            ->setLogin($authorDetails['login'])
            ->setPlatform(self::PLATFORM_NAME);
        $this->entityManager->persist($author);
        $this->entityManager->flush();
        return $author;
    }
}
